CtA_Hawk = LibStub("AceAddon-3.0"):NewAddon("CtA_Hawk", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CtA_Hawk", true)
local LSM = LibStub("LibSharedMedia-3.0")

LSM:Register("sound", "War Drums", [[Sound\Event Sounds\Event_wardrum_ogre.wav]])

local DB_VERSION = 2

local UPDATEPERIOD, elapsed = 15, 0

CtA_Hawk.display = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("CtA Hawk", {type = "data source", label=L["CtA"], text = L["Unknown"]})

CtA_Hawk.updateframe = CreateFrame("frame")

local dungeonGroups = {}
--CtA_Hawk.dungeonGroups = dungeonGroups

local RARE = LFG_ROLE_SHORTAGE_RARE
local UNCOMMON = LFG_ROLE_SHORTAGE_UNCOMMON
local PLENTIFUL = LFG_ROLE_SHORTAGE_PLENTIFUL
local BALANCED = -1
local UNKNOWN = 0

local shortageColor = {
	[RARE] = "|cFFFF7F00",
	[UNCOMMON] = "|cFFFFFF00",
	[PLENTIFUL] = "|cFF7FFF00",
	[BALANCED] = "|cFF00FFFF",
	[UNKNOWN] = "|cFF555555",
}

local shortageName = {
	[RARE] = shortageColor[RARE]..L["Rare"],
	[UNCOMMON] = shortageColor[UNCOMMON]..L["Uncommon"],
	[PLENTIFUL] = shortageColor[PLENTIFUL]..L["Plentiful"],
	[BALANCED] = shortageColor[BALANCED]..L["Balanced"],
	[UNKNOWN] = shortageColor[UNKNOWN]..L["Unknown"],
}

local roleCounts = {}
--CtA_Hawk.roleCounts = roleCounts

local availableRoles = {}

local roleIcons = {
	Tank = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:0:19:20:39|t",
	Healer = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:0:19|t",
	DPS = "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:20:20:0:0:64:64:20:39:20:39|t",
}
roleIcons.Balance = roleIcons.Tank..roleIcons.Healer..roleIcons.DPS

function CtA_Hawk:OnInitialize()

	if CtA_HawkOptions ~= nil then 
		if (CtA_HawkOptions.version or 0) < DB_VERSION then CtA_HawkOptions = {version = DB_VERSION} end
	end

	availableRoles["Tank"], availableRoles["Healer"], availableRoles["DPS"] = UnitGetAvailableRoles("player")
	local roleOptions = {None = L["None"]}
	if availableRoles["Tank"] then roleOptions.Tank = L["Tank"] end
	if availableRoles["Healer"] then roleOptions.Healer = L["Healer"] end
	if availableRoles["DPS"] then roleOptions.DPS = L["DPS"] end

	self.options = {
		name = L["CtA Hawk: Options"],
		type = "group",
		args = {
			settings = {
				name = "settings",
				type = "group",
				args = {
					enableAlerts = {
						type = "toggle",
						name = L["Enable Alerts"],
						desc = L["Check this box to enable alerts. Configure the alert rules for each role below."],
						set = function(info,val)
							CtA_Hawk.db.profile.enableAlerts = val
						end,
						get = function(info) return CtA_Hawk.db.profile.enableAlerts end,
						order = 0,
					},
					quietInGroup = {
						type = "toggle",
						name = L["Quiet In Group"],
						desc = L["Check this box to suppress alerts when you are in a group."],
						disabled = function() return not CtA_Hawk.db.profile.enableAlerts end,
						set = function(info,val)
							CtA_Hawk.db.profile.quietInGroup = val
						end,
						get = function(info) return CtA_Hawk.db.profile.quietInGroup end,
						order = 0,
					},
					enableAutoQueue = {
						type = "toggle",
						name = L["Enable Auto-Queue"],
						desc = L["EnableAutoQueueNote1"].."\n|cFFFFCC00"..L["EnableAutoQueueNote2"],
						descStyle = "inline",
						disabled = function() return GetLFGMode() ~= nil or GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 end,
						width = "full",
						set = function(info,val)
							if val then
								CtA_Hawk:EnableAutoQueue()
							else
								CtA_Hawk:DisableAutoQueue()
							end
						end,
						get = function(info) return CtA_Hawk.autoQueueEnabled end,
						order = 1,
					},
					general = {
						type = "group",
						name = L["General"],
						order = 1,
						args = {
							ldbDisplay = {
								type = "group",
								name = L["LDB Display Options"],
								inline = true,
								order = 2,
								args = {
									shortLabel = {
										type = "toggle",
										name = L["Short Label"],
										desc = L["Display \"CtA\" on the LDB text instead of \"Call To Arms\""],
										order = 1,
										set = function(info, val)
											CtA_Hawk.db.profile.shortLabel = val
											CtA_Hawk:RefreshLDB()
										end,
										get = function()
											return CtA_Hawk.db.profile.shortLabel
										end,
									},
									shortText = {
										type = "toggle",
										name = L["Short Text"],
										desc = L["Display \"T\", \"H\" and \"D\" on the LDB text instead of \"Tank\", \"Healer\" and \"DPS\""],
										order = 2,
										set = function(info, val)
											CtA_Hawk.db.profile.shortText = val
											CtA_Hawk:RefreshLDB()
										end,
										get = function()
											return CtA_Hawk.db.profile.shortText
										end,
									},
								},
							},
						},
					},
					queues = {
						type = "group",
						name = L["Dungeon Groups"],
						desc = L["Select the dungeon queues for which you want to monitor role status."],
						order = 2,
						args = {},
					},
					Tank = {
						type = "group",
						name = roleIcons.Tank..L["Tank"],
						order = 3,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.Tank,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Tank.autoQueue[key] = val
									if val then CtA_Hawk:RoleAutoQueueEnabled("Tank", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Tank.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Tank.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled("Tank", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Tank.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.Tank.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Tank.soundFile
								end,
							},
						},
					},
					Healer = {
						type = "group",
						name = roleIcons.Healer..L["Healer"],
						order = 4,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.Healer,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Healer.autoQueue[key] = val
									if val then CtA_Hawk:RoleAutoQueueEnabled("Healer", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Healer.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.Healer.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled("Healer", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.Healer.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = LSM:HashTable("sound"),
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.Healer.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Healer.soundFile
								end,
							},
						},
					},
					DPS = {
						type = "group",
						name = roleIcons.DPS..L["DPS"],
						order = 5,
						args = {
							autoQueue = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Auto-Queue When'],
								desc = L["AutoQueueNote1"].."\n|cFFFFCC00"..L["AutoQueueNote2"],
								disabled = not availableRoles.DPS,
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 1,
								set = function(info, key, val)
									CtA_Hawk.db.profile.DPS.autoQueue[key] = val
									if val then CtA_Hawk:RoleAutoQueueEnabled("DPS", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.DPS.autoQueue[key]
								end,
							},
							notify = {
								type = 'multiselect',
								name = '|cFFFFCC00'..L['Notify When'],
								desc = L["Alerts you when this role enters this status. The options below control the type of notification."],
								values = {
									[RARE] = shortageName[RARE],
									[UNCOMMON] = shortageName[UNCOMMON],
									[PLENTIFUL] = shortageName[PLENTIFUL],
								},
								order = 2,
								set = function(info, key, val)
									CtA_Hawk.db.profile.DPS.notify[key] = val
									if val then CtA_Hawk:RoleMonitoringEnabled("DPS", key) end
								end,
								get = function(info, key)
									return CtA_Hawk.db.profile.DPS.notify[key]
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.textNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when this role enters one of the statuses selected above."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.chatNotify = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when this role enters one of the statuses selected above."],
								values = AceGUIWidgetLSMlists.sound,
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.DPS.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.DPS.soundFile
								end,
							},
						},
					},
					Balance = {
						type = "group",
						name = "|T:20:20|t"..L["Balance"],
						order = 6,
						args = {
							autoQueue = {
								type = "select",
								name = L["Auto-Queue As"],
								desc = L["Automatically enter the queue for the following role when all roles are in balance."],
								order = 1,
								width = "full",
								values = roleOptions,
								set = function (_, val)
									CtA_Hawk.db.profile.Balance.autoQueue = val
									if val ~= "None" then CtA_Hawk:RoleAutoQueueEnabled(val, BALANCED) end
								end,
								get = function ()
									return CtA_Hawk.db.profile.Balance.autoQueue
								end,
							},
							text = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Text Alert"],
								desc = L["Display message on screen when all roles are in balance."],
								descStyle = "inline",
								order = 4,
								set = function(info, val)
									CtA_Hawk.db.profile.Balance.textNotify = val
									if val and not CtA_Hawk.db.profile.Balance.chatNotify and CtA_Hawk.db.profile.Balance.soundFile == "None" then CtA_Hawk:RoleMonitoringEnabled("Balance", BALANCED) end
								end,
								get = function()
									return CtA_Hawk.db.profile.Balance.textNotify
								end,
							},
							chat = {
								type = "toggle",
								name = "|cFFFFCC00"..L["Chat Alert"],
								desc = L["Display message in chat window when all roles are in balance."],
								descStyle = "inline",
								order = 5,
								set = function(info, val)
									CtA_Hawk.db.profile.Balance.chatNotify = val
									if val and not CtA_Hawk.db.profile.Balance.textNotify and CtA_Hawk.db.profile.Balance.soundFile == "None" then CtA_Hawk:RoleMonitoringEnabled("Balance", BALANCED) end
								end,
								get = function()
									return CtA_Hawk.db.profile.Balance.chatNotify
								end,
							},
							soundFile = {
								type = "select",
								dialogControl = "LSM30_Sound",
								name = L["Audible Alert"],
								desc = L["Select a sound file to be played when all roles are in balance."],
								values = AceGUIWidgetLSMlists.sound,
								get = function()
									return current
								end,
								set = function(self,key)
									current = key
									if val and not CtA_Hawk.db.profile.Balance.chatNotify and not CtA_Hawk.db.profile.Balance.textNotify then CtA_Hawk:RoleMonitoringEnabled("Balance", BALANCED) end
								end,
								order = 7,
								set = function(info, val)
									CtA_Hawk.db.profile.Balance.soundFile = val
								end,
								get = function()
									return CtA_Hawk.db.profile.Balance.soundFile
								end,
							},
						},
					},
				},
			},
		},
	}
	
	self.defaults = {
		profile = {
			enableAlerts = true,
				quietInGroup = true,
			shortLabel = false,
			shortText = false,
			queues = {
				["auto"] = true,
			},
			Tank = {
				autoQueue = {},
				notify = availableRoles.Tank and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
				soundFile = "War Drums",
			},
			Healer = {
				autoQueue = {},
				notify = availableRoles.Healer and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
				soundFile = "War Drums",
			},
			DPS = {
				autoQueue = {},
				notify = availableRoles.DPS and {[RARE] = true} or {},
				textNotify = true,
				chatNotify = true,
 				soundFile = "War Drums",
			},
			Balance = {
				autoQueue = "None",
				textNotify = false,
				chatNotify = false,
 				soundFile = "None",
			}
		}
	}
	
	self.db = LibStub("AceDB-3.0"):New("CtA_HawkOptions",self.defaults)
	
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CtA_Hawk", self.options)
	
	self.settingsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CtA_Hawk", L["CtA Hawk"], nil, "settings")  
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CtA_Hawk", L["Profiles"], L["CtA Hawk"], "profiles")
	
	self.updateframe:SetScript("OnUpdate",CtA_Hawk.UpdateFrameUpdate)

	self:RegisterEvent("LFG_UPDATE_RANDOM_INFO")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","DisableAutoQueue")
	self:RegisterEvent("LFG_UPDATE","DisableAutoQueue")
end

function CtA_Hawk:DisableAutoQueue()
	if self.autoQueueEnabled then
		self.autoQueueEnabled = false
		print("|cFFFFCC00"..L["Auto-Queue Disabled!"])
	end
	LibStub("AceConfigRegistry-3.0"):NotifyChange("CtA_Hawk");
end

function CtA_Hawk:EnableAutoQueue()
	self.autoQueueEnabled = true
	print("|cFFFFCC00"..L["Auto-Queue Enabled!"])
	if self.db.profile.Tank.autoQueue[RARE] then self:RoleAutoQueueEnabled("Tank",RARE) end
	if self.db.profile.Tank.autoQueue[UNCOMMON] then self:RoleAutoQueueEnabled("Tank",UNCOMMON) end
	if self.db.profile.Tank.autoQueue[PLENTIFUL] then self:RoleAutoQueueEnabled("Tank",PLENTIFUL) end

	if self.db.profile.Healer.autoQueue[RARE] then self:RoleAutoQueueEnabled("Healer",RARE) end
	if self.db.profile.Healer.autoQueue[UNCOMMON] then self:RoleAutoQueueEnabled("Healer",UNCOMMON) end
	if self.db.profile.Healer.autoQueue[PLENTIFUL] then self:RoleAutoQueueEnabled("Healer",PLENTIFUL) end

	if self.db.profile.DPS.autoQueue[RARE] then self:RoleAutoQueueEnabled("DPS",RARE) end
	if self.db.profile.DPS.autoQueue[UNCOMMON] then self:RoleAutoQueueEnabled("DPS",UNCOMMON) end
	if self.db.profile.DPS.autoQueue[PLENTIFUL] then self:RoleAutoQueueEnabled("DPS",PLENTIFUL) end

	if self.db.profile.Balance.autoQueue ~= "None" then self:RoleAutoQueueEnabled(self.db.profile.Balance.autoQueue,BALANCED) end
end

function CtA_Hawk:OnLFGInitialize()

	local t = {
		["auto"] = {
			type = "toggle",
			name = "|cFFFFCC00"..L["Auto-recommend"],
			set = function(info, val)
				CtA_Hawk.db.profile.queues = {
					["auto"] = val,
					[GetRandomDungeonBestChoice()] = true,
				}
			end,
			get = function()
				return CtA_Hawk.db.profile.queues.auto
			end,
			order = 0,
		}
	}
	
	for i=1, GetNumRandomDungeons() do
		local groupID, groupName = GetLFGRandomDungeonInfo(i)
		local eligible = GetLFGRoleShortageRewards(groupID,1)
		dungeonGroups[groupID] = {
			name = groupName,
			eligible = eligible,
			Tank = UNKNOWN,
			Healer = UNKNOWN,
			DPS = UNKNOWN,
		}
		t["Dungeon"..groupID] = {
			type = "toggle",
			name = strsub(groupName, 8),
			disabled = function() return CtA_Hawk.db.profile.queues.auto end,
			hidden = function() return not dungeonGroups[groupID].eligible end,
			set = function(info, val)
				CtA_Hawk.db.profile.queues[groupID] = val
			end,
			get = function()
				return CtA_Hawk:IsDungeonGroupMonitored(groupID)
			end,
			order = 10000 - groupID,
		}
	end
	self.options.args.settings.args.queues.args = t

	self.LFGInitialized = true

	RequestLFDPlayerLockInfo()

	CtA_Hawk:RefreshLDB()

end

function CtA_Hawk.UpdateFrameUpdate(self, elap)

	if not CtA_Hawk.LFGInitialized then
		elapsed = elapsed + elap
		if elapsed < .5 then return end
		elapsed = 0
		if GetNumRandomDungeons() > 0 then 
			CtA_Hawk:OnLFGInitialize()
		end
		return
	end
	
	elapsed = elapsed + elap
	if elapsed < UPDATEPERIOD then return end
	elapsed = 0

	RequestLFDPlayerLockInfo()

end

function CtA_Hawk:IsDungeonGroupMonitored(dungeonID)
	if CtA_Hawk.db.profile.queues[dungeonID] and not CtA_Hawk.db.profile.queues.auto then return true end
	if CtA_Hawk.db.profile.queues.auto and dungeonID == GetRandomDungeonBestChoice() then return true end
	return false
end

function CtA_Hawk:RoleMonitoringEnabled(role, status)

	if not self.LFGInitialized then return end

	for groupID, groupData in pairs(dungeonGroups) do
		if CtA_Hawk:IsDungeonGroupMonitored(groupID) then
			if groupData[role] == status then self:ProcessDungeonStatusChange(groupID, role, status) end
		end
	end
	
end

function CtA_Hawk:RoleAutoQueueEnabled(role, status)

	if not self.LFGInitialized then return end
	if not self.autoQueueEnabled then return end
	if GetLFGMode() ~= nil or GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then return end
	
	for groupID, groupData in pairs(dungeonGroups) do
		if CtA_Hawk:IsDungeonGroupMonitored(groupID) then
			if groupData[status == BALANCED and "Balance" or role] == status then 
				self:EnterQueue(groupID, role)
				break
			end
		end
	end
	
end

function CtA_Hawk:LFG_UPDATE_RANDOM_INFO ()

	local dungeonGroup

	if not self.LFGInitialized then return end
	
	roleCounts = {
		Tank = { [RARE] = 0, [UNCOMMON] = 0, [PLENTIFUL] = 0, [UNKNOWN] = 0 },
		Healer = { [RARE] = 0, [UNCOMMON] = 0, [PLENTIFUL] = 0, [UNKNOWN] = 0 },
		DPS = { [RARE] = 0, [UNCOMMON] = 0, [PLENTIFUL] = 0, [UNKNOWN] = 0 },
	}

	for groupID, groupData in pairs(dungeonGroups) do
		
		dungeonGroup = {
			name = groupData.name,
			Tank = PLENTIFUL,
			Healer = PLENTIFUL,
			DPS = PLENTIFUL,
		}

		for shortageID=1,3 do
			
			local eligible, forTank, forHealer, forDPS, itemCount, money, xp = GetLFGRoleShortageRewards(groupID, shortageID);
			
			if eligible then dungeonGroup.eligible = true end

			if forTank then dungeonGroup.Tank = shortageID end
			if forHealer then dungeonGroup.Healer = shortageID end
			if forDPS then dungeonGroup.DPS = shortageID end

			if itemCount > 0 or money > 0 or xp > 0 then dungeonGroup.reward = true end

		end
		
		if CtA_Hawk:IsDungeonGroupMonitored(groupID) then
			roleCounts.Tank[dungeonGroup.Tank] = roleCounts.Tank[dungeonGroup.Tank] + 1
			roleCounts.Healer[dungeonGroup.Healer] = roleCounts.Healer[dungeonGroup.Healer] + 1
			roleCounts.DPS[dungeonGroup.DPS] = roleCounts.DPS[dungeonGroup.DPS] + 1
			
			if dungeonGroups[groupID].Tank ~= dungeonGroup.Tank then self:ProcessDungeonStatusChange(groupID, "Tank", dungeonGroup.Tank) end
			if dungeonGroups[groupID].Healer ~= dungeonGroup.Healer then self:ProcessDungeonStatusChange(groupID, "Healer", dungeonGroup.Healer) end
			if dungeonGroups[groupID].DPS ~= dungeonGroup.DPS then self:ProcessDungeonStatusChange(groupID, "DPS", dungeonGroup.DPS) end

			if dungeonGroup.Tank == PLENTIFUL and dungeonGroup.Healer == PLENTIFUL and dungeonGroup.DPS == PLENTIFUL
					and (dungeonGroups[groupID].Tank ~= PLENTIFUL or dungeonGroups[groupID].Healer ~= PLENTIFUL or dungeonGroups[groupID].DPS ~= PLENTIFUL) then
				self:ProcessDungeonStatusChange(groupID, "Balance", BALANCED)
			end
			
		end
	
		dungeonGroups[groupID] = dungeonGroup

	end

end

function CtA_Hawk:ProcessDungeonStatusChange(groupID, role, newStatus)

	self:RefreshLDB()
	
	if CtA_Hawk.db.profile.quietInGroup and GetNumPartyMembers() + GetNumRaidMembers() > 0 then return end

	if role == "Balance" then
		if not newStatus == BALANCED then return end
	elseif not self.db.profile[role].notify[newStatus] then 
		return 
	end
	
	if self.db.profile[role].textNotify then
		--print(groupID, role, newStatus)
		RaidNotice_AddMessage( RaidBossEmoteFrame, L["Call To Arms: "]..roleIcons[role]..shortageColor[newStatus]..L[role].."|r - "..dungeonGroups[groupID].name, ChatTypeInfo["RAID_BOSS_EMOTE"] )
	end
	
	if self.db.profile[role].chatNotify then
		ChatFrame1:AddMessage( L["Call To Arms: "]..roleIcons[role]..shortageColor[newStatus]..L[role].."|r - "..dungeonGroups[groupID].name, 1, .8, 0, 1 )
	end
	
	if self.db.profile[role].soundFile ~= "" and self.db.profile[role].soundFile ~= "None" then
		PlaySoundFile(LSM:Fetch("sound",self.db.profile[role].soundFile),"Master")
	end
	
	if self.autoQueueEnabled then
		if role == "Balance" then
			if self.db.profile.Balance.autoQueue ~= "None" then self:EnterQueue(groupID, self.db.profile.Balance.autoQueue) end
		else
			if self.db.profile[role].autoQueue[newStatus] then self:EnterQueue(groupID, role) end
		end
	end
end

function CtA_Hawk:EnterQueue(groupID, role)
	local LFGRoles = {}
	LFGRoles.Leader, LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS = GetLFGRoles()
	if role then
		LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS = false, false, false
		LFGRoles[role] = true
		self.autoQueue = nil
	end
	SetLFGRoles(LFGRoles.Leader, LFGRoles.Tank, LFGRoles.Healer, LFGRoles.DPS)
	if groupID then 
		LFDQueueFrame.type = groupID
	elseif not LFDQueueFrame.type then
		LFDQueueFrame.type = GetRandomDungeonBestChoice()
	end
	UIDropDownMenu_SetSelectedValue(LFDQueueFrameTypeDropDown, LFDQueueFrame.type)
	SetLFGDungeon(groupID)
	LFDQueueFrameRandom_UpdateFrame();
	JoinLFG()
end

function CtA_Hawk:RefreshLDB()
	self.display.label = self.db.profile.shortLabel and L["CtA"] or L["Call To Arms"] 
	self.display.icon = "Interface\\ICONS\\Achievement_Arena_2v2_3"

	local text = ""
	if roleCounts.Tank == nil then
		text = "|cFF555555"..L["Unknown"]
	elseif roleCounts.Tank[RARE] + roleCounts.Tank[UNCOMMON] + roleCounts.Healer[RARE] + roleCounts.Healer[UNCOMMON] + roleCounts.DPS[RARE] + roleCounts.DPS[UNCOMMON] == 0 then
		text = shortageName[BALANCED]
	else
		for role in pairs(availableRoles) do
			for i=1, 3 do
				if roleCounts[role][i] > 0 then
					text = text..shortageColor[i]..(self.db.profile.shortText and L[role.."Short"] or L[role]) .. " " --..(roleCounts[role][i] > 1 and "("..roleCounts[role][i]..") " or " ")
					break
				end
			end
		end
		if text == "" then text = "|cFF555555"..L["Unknown"] end
	end
	self.display.text = text
end

function CtA_Hawk.display:OnEnter()

	local tooltip = LibStub("LibQTip-1.0"):Acquire("CtA_HawkTooltip", 6, "LEFT", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER")
	local checkmark = "|TInterface\\BUTTONS\\UI-CheckBox-Check:20:20|t"
	local goldicon = "|TInterface\\MONEYFRAME\\UI-GoldIcon:16:16|t"
	self.tooltip = tooltip 
	tooltip:AddHeader(L["Dungeon Group"], L["MonitoredHeader"],L["RewardHeader"], L["Tank"], L["Healer"], L["DPS"])
	for i=GetNumRandomDungeons(), 1, -1 do
		local groupID = GetLFGRandomDungeonInfo(i)
		local groupData = dungeonGroups[groupID]
		if groupData.eligible then
			tooltip:AddLine(
				"|cFFFFCC00"..groupData.name, 
				(CtA_Hawk:IsDungeonGroupMonitored(groupID) and checkmark or ""),
				(groupData.reward and goldicon or ""),
				shortageName[groupData.Tank], 
				shortageName[groupData.Healer], 
				shortageName[groupData.DPS].." "
			)
		end
	end
	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

function CtA_Hawk.display:OnLeave()
   -- Release the tooltip
   LibStub("LibQTip-1.0"):Release(self.tooltip)
   self.tooltip = nil
end

function CtA_Hawk.display:OnClick(button)
	if IsMouseButtonDown(button) then return end
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory(CtA_Hawk.settingsFrame)
	end
	if button == "LeftButton" then
		if IsControlKeyDown() then
			if IsShiftKeyDown() then
				CtA_Hawk:EnterQueue()
			elseif CtA_Hawk.autoQueueEnabled then 
				CtA_Hawk:DisableAutoQueue() 
			else 
				CtA_Hawk:EnableAutoQueue() 
			end
		else
			ToggleLFDParentFrame()
		end
	end
end